#include <stdio.h>
#include "arbre.h"
#include <string.h>

/* Creation d'un noeud
 * alloue la memoire et ajoute le caractere
 * */
arbre arbre_creerNoeud(char lettre)
{
    noeud *n = NULL;
    if ((n = (noeud *) malloc(sizeof(noeud))) == NULL) {
	fprintf(stderr, "erreur dans creerNoeud\n");
	return NULL;
    }
    n->c = lettre;
    n->fg = n->fd = n->fils = NULL;
    return n;
}

/* Initialisation de l'arbre
 * avec le prefixe vide a la racine
 * retourne NULL si l'*/
arbre arbre_init(void)
{
    arbre a = NULL;
    a = (arbre) malloc(sizeof(noeud));
    a->info = (infos *) malloc(sizeof(infos));
    a->info->nbocc = 0;
    a->info->positions = NULL;
    a->fg = a->fd = a->fils = 0;
    a->c = 0;
    return a;
}

/* Parcours recurssif de l'arbre
 * pour ajouter le mot dans l'arbre 
 * retourne 1 si tout c'est bien pass et 0 sinon
 * */
int arbre_ajoutMot(arbre a, char *mot, int i, long int position)
{
    if (mot[i] == 0) {
	if (a->info != NULL) {
	    a->info->nbocc += 1;
	    liste_ajout(&(a->info->positions), position);
	    return 1;
	} else {
	    while (a->fg != NULL) {
		if (a->info != NULL) {
		    a->info->nbocc += 1;
		    liste_ajout(&(a->info->positions), position);
		    return 1;
		}
		a = a->fg;
	    }
	    a->fg = arbre_creerNoeud(0);
	    a->fg->info = (infos *) malloc(sizeof(infos));
	    a->fg->info->nbocc = 1;
	    liste_ajout(&(a->fg->info->positions), position);
	    return 1;
	}
	return 1;
    }
    if (*(mot + i) == a->c)
	return arbre_ajoutMot(a->fils, mot, (i + 1), position);
    if ((a->fd != NULL) && (*(mot + i) > a->c))
	return arbre_ajoutMot(a->fd, mot, i, position);
    if ((a->fg != NULL) && (*(mot + i) < a->c))
	return arbre_ajoutMot(a->fg, mot, i, position);
    if ((a->fd == NULL) || (a->fg == NULL) || (a->fils == NULL)) {
	int taille = strlen(mot);
	if (mot[i] < a->c) {
	    a->fg = arbre_creerNoeud(mot[i]);
	    if (a->fg == NULL) {
		fprintf(stderr, "erreur dans arbre_ajout\n");
		arbre_detruire(&a);
		return 0;
	    }
	    a = a->fg;
	    i++;
	} else {
	    a->fd = arbre_creerNoeud(mot[i]);
	    if (a->fd == NULL) {
		fprintf(stderr, "erreur dans arbre_ajout\n");
		arbre_detruire(&a);
		return 0;
	    }
	    a = a->fd;
	    i++;
	}
	while (i < taille) {
	    a->fils = arbre_creerNoeud(mot[i]);
	    if (a->fils == NULL) {
		fprintf(stderr, "erreur dans arbre_ajout\n");
		arbre_detruire(&a);
		return 0;
	    }
	    a = a->fils;
	    i++;
	}
	a->fils = arbre_creerNoeud(0);
	a->fils->info = (infos *) malloc(sizeof(infos));
	a->fils->info->nbocc = 1;
	liste_ajout(&(a->fils->info->positions), position);
	return 1;
    }
    return 0;
}


/* Ajout d'un mot dans l'arbre
 * retourne 1 si tout c'est bien pass et 0 sinon */
int arbre_ajout(arbre a, char *mot, long int position)
{
    if (*mot == 0)
	return 1;

    int i = 0;
    int taille = strlen(mot);
    if (a->fils == NULL) {

	while (i < taille) {
	    a->fils = arbre_creerNoeud(*(mot + i));
	    if (a->fils == NULL) {
		fprintf(stderr, "erreur dans arbre_ajout\n");
		arbre_detruire(&a);
		return 0;
	    }
	    a = a->fils;
	    i++;
	}
	a->fils = arbre_creerNoeud(0);
	a->fils->info = (infos *) malloc(sizeof(infos));
	a->fils->info->nbocc = 1;
	liste_ajout(&(a->fils->info->positions), position);
	return 1;
    }
    i = 0;
    return arbre_ajoutMot(a->fils, mot, i, position);
}

/* Libere la memoire de l'arbre et la liste des occurences
 * des mots
 * */
void arbre_detruire(arbre * a)
{
    if ((*a) == NULL)
	return;
    if ((*a)->info != NULL) {
	liste_detruire((*a)->info->positions);
	free((*a)->info);
    }
    arbre_detruire(&((*a)->fils));
    arbre_detruire(&((*a)->fd));
    arbre_detruire(&((*a)->fg));
    free(*a);
}

/* Recherche un mot dans l'arbre 
 * */
noeud *arbre_recherche_(arbre a, char *mot, int position)
{
    if (a == NULL)
	return NULL;
    if ((*(mot + position)) == 0)
	return a;
    if (*(mot + position) == a->c)
	return arbre_recherche_(a->fils, mot, position + 1);
    if (*(mot + position) > a->c)
	return arbre_recherche_(a->fd, mot, position);
    if (*(mot + position) < a->c)
	return arbre_recherche_(a->fg, mot, position);
    return NULL;
}

/* Affiche les phrases ou se trouvent le mot
 * */
int arbre_affichePhrases(arbre a, char *mot, char *texte)
{
    if (a == NULL)
	return 1;
    arbre temp = NULL;
    liste templ;
    int lettre;
    FILE *fichier;
    if ((temp = arbre_recherche(a->fils, mot)) == NULL) {
	fprintf(stderr,
		"Le mot n'est pas dans le texte impossible d'afficher les phrases\n");
	return 2;
    }
    if ((fichier = fopen(texte, "r")) == NULL) {
	fprintf(stderr, "Impossible d'ouvrir le fichier %s\n", texte);
	return 2;
    }
    templ = temp->info->positions;
    while (templ != NULL) {
	fseek(fichier, templ->v, SEEK_SET);
	lettre = fgetc(fichier);
	while ((lettre != '.') && (lettre != '!') && (lettre != '?')
	       && (lettre != EOF)) {
	    printf("%c", lettre);
	    lettre = fgetc(fichier);
	}
	templ = templ->suiv;
    }
    printf("\n");

    fclose(fichier);
    return 0;
}

/* Appel la fonction recherche et affiche le nombre d'occurence
 * du mot et ses positions dans le texte
 * */
int arbre_afficheOccurences(arbre a, char *mot)
{
    if (a == NULL)
	return 1;
    arbre temp = NULL;
    temp = arbre_recherche(a, mot);
    liste templ = temp->info->positions;
    if (temp == NULL) {
	printf("Le mot %s n'est pas dans l'arbre\n", mot);
	return 2;
    }
    printf("Nombre d'occurences: %d\n", temp->info->nbocc);
    while (templ != NULL) {
	printf("%ld ", templ->v);
	templ = templ->suiv;
	printf("\n");
    }
    return 0;
}

/* Affichage de tous les mots de l'arbre
 * */
void arbre_afficheMots(arbre a, char *buf, int i)
{
    if (a == NULL)
	return;
    arbre_afficheMots(a->fg, buf, i);
    if (a->c != 0) {
	buf[i] = a->c;
	i++;
    } else {
	buf[i] = '\0';
	i++;
	printf("%s \n", buf);
    }
    arbre_afficheMots(a->fils, buf, i);
    i--;
    arbre_afficheMots(a->fd, buf, i);
    i--;
}
